;********************************************************************
;                         Tomasz Jaboski                          *
;                 Mikrokontrolery PIC16F8x w praktyce               *
;                      Wydawnictwo BTC, www.btc.pl                  *
;********************************************************************

;plik rdowy termometru cyfrowego z ukadem DS1620 i usatwianiem termostatu
;temperatura dolna +27C
;temperatura gorna +28C
;temeratura jest odczytywana co ok 200msek i wyswietlana na wywietlaczu 
;alfanumerycznym LCD
;schemat poacze moduu AVT873 z ukadem DS1620 i wywietlaczem pokazany
;jest na rysunku 8.40
#include 16f84.h; plik 16f84 musi by w katalogu roboczym
temp 		equ 0x10
temp1		equ 0x11
i		equ 0x12
zbit		equ 0x13
dana		equ 0x14
jedn		equ 0x15
dzies		equ 0x16
ul		equ 0x17
#define bit9	zbit,0
#define znak	zbit,1
#define	CLK	porta,2
#define RST	porta,3
#define DQ	porta,4
;deklaracje wywietlacza LCD
msek		equ 17h
TMP		equ 18h
adres		equ 19h
#define E	porta,1
#define RS	porta,0
;********************************************************************
		__config 	h'3ff1'
		org 0

		BSF RP0		;bank0
		CLRF trisa	;wszystkie linie wyjciowe
		CLRF trisb	;j.w.
		BCF RP0
		bcf 0b,7
		CALL iniLCD	;inicjalizacja wywietlacza LCD
;zapisanie sowa konfiguracji- tryb pracy z CPU i pomiary cige
		BSF RST		;RST=1
		MOVLW 0ch	;kod zapisu konfiguracji
		BCF bit9	;dana 8bitowa
		CALL zapis_ds	;wpisanie kodu
		BCF bit9
		MOVLW .2	;tryb pomiarw cigych i praca z CPU
		CALL zapis_ds
		BCF RST		;RST=0

		MOVLW .20	
		CALL delay_msek
		BSF RST
		MOVLW 0eeh	;start przetwarzania
		CALL zapis_ds
		BCF RST

;fragment wpisywania nastaw termostatu - temperatury grnej (28C)
;i temperatury dolnej(27C). 

		MOVLW .27	;temperatura
		BCF znak	;znak
		CALL zapis_tl	;zapis temperatury dolnej
		MOVLW .28	;temperatura
		BCF znak	
		CALL zapis_th	;zapis temperatury gornej

;ptla termometru
	
termometr	MOVLW 80h	;adres 00
		CALL wrcmd
		
		CALL odczyt_temp;odczytanie temperatury z DS1620
		CALL wys_temp	;konwersja  wartoci rejestru temperatury
		MOVF dana,w	
		CALL _konw	;konwersja hex->dec

		MOVLW '-'
		BTFSC znak
		GOTO et_wys_t 
		MOVLW '+'

et_wys_t	CALL wrdata
		MOVF dzies,w
		ADDLW 30h	;kod ascii dziesitek
		CALL wrdata

		MOVF jedn,w
		ADDLW 30h	;kod ascii jedynek
		CALL wrdata
	
		MOVLW '.'	;kropka
		CALL wrdata

		MOVF ul,w
		ADDLW 0x30	;kod ascii uamka
		CALL wrdata	;wywietlenie uamka

		MOVLW 'C'
		CALL wrdata	;wywietlenie C
		MOVLW .200
		CALL delay_msek
		
		GOTO termometr
		

		SLEEP

;********************************************************************
;procedura odczytu temperatury.Temperatura w rejestrze W znak w bicie znak
;********************************************************************
odczyt_temp	BSF RST		;RST=1
		MOVLW 0aah	;kod odczytu temperatury
		BCF bit9	;dana 8bitowa
		CALL zapis_ds	;wpisanie kodu
		BSF bit9
		CALL odczyt_ds	;odczytanie rejestru temperatury- temperatura 
				;w rej W, 9-ty bit znaku w bicie znak
		BCF RST		;RST=0
		RETURN
;********************************************************************
;procedura zapisu do termostatu temperatury dolnej
;temperatura w rejestrze W (cae stopnie)
;znak  wbicie znak
;********************************************************************
zapis_tl	CALL konw_term	;konwersja do postaci DS1620
		MOVWF jedn	;czasowe przechowanie zawartoci W
		BSF RST		;RST=1
		MOVLW .2	;kod zapisu Tl
		BCF bit9	;dana 8bitowa
		CALL zapis_ds	;wpisanie kodu
		BSF bit9	;9- bitowa temperatura
		MOVF jedn,w	;do W temperatura
		CALL zapis_ds
		BCF RST		;RST=0
		MOVLW .20	
		CALL delay_msek
		RETURN

;********************************************************************
;procedura zapisu do termostatu temperatury gornej
;temperatura w rejestrze W (cae stopnie)
;znak  wbicie znak
;********************************************************************
zapis_th	CALL konw_term	;konwersja do postaci DS1620
		MOVWF jedn	;czasowe przechowanie zawartoci W
		BSF RST		;RST=1
		MOVLW .1	;kod zapisu Tl
		BCF bit9	;dana 8bitowa
		CALL zapis_ds	;wpisanie kodu
		BSF bit9	;9- bitowa temperatura
		MOVF jedn,w	;do W temperatura
		CALL zapis_ds
		
		BCF RST	
		MOVLW .20	;RST=0	
		CALL delay_msek	

		RETURN
;*********************************************************************
		
 
;********************************************************************
;procedura odczytania danych z DS1620. Dana w rejestrze W a znak w 
;bicie znak. Jeeli maj by odczytane dane 9bitowe to bit9 musi by jedynk
;********************************************************************


odczyt_ds	CLRF  temp
		CLRF  temp1
		CLRF  i		 ;for(i=0i<=7;i++)
et_od1		MOVLW .8
		SUBWF i,W
		BTFSC C
		GOTO  et_od4
		BCF   C		 ;temp=temp>>1
		RRF   temp,1
		BSF   RP0   	 ;bank1
		MOVLW b'00010000';RA4 (DQ)wejsciowy
		MOVWF trisa
		BCF   RP0	;bank0
		BCF   CLK	;CLK=0
		BTFSS DQ	;if(DQ1==1)
		GOTO  et_od2
		MOVLW .128	;temp1=0x80
		MOVWF temp1
		GOTO  et_od3
et_od2		CLRF  temp1	;temp1=0
et_od3		MOVF  temp1,W	;temp=temp|temp1
		IORWF temp,1
		BCF   RP0
		BSF   CLK	;CLK=1
		INCF  i,1
		GOTO  et_od1
et_od4		MOVF  temp,W	;dana=temp
		MOVWF dana
		BTFSS bit9	;if(bit9==1)
		GOTO  et_od5
		BCF   RP0
		BCF   CLK	;CLK=0
		BCF   znak
		BTFSC DQ	;znak=DQ
		BSF   znak
		BSF   CLK	;CLK=1
et_od5		BSF RP0		;bank1
		CLRF trisa	;RA4 wyjsciowy
		BCF RP0		;bank0
	
		RETURN


;********************************************************************
;procedura zapisu do DS1620. Jeeli dana ma by 9-bitowa to bit znak9 
;musi mie warto 1. Znak temperatury (9- ty bit) umieszczany jest 
;wtedy w bicie znak. 8 modszych bitw danych umieszczonych jest w rej.W
;********************************************************************
zapis_ds	MOVWF temp	;temp=W
		CLRF  i		;for(i=0;i<=7;i++)
et_zapis1	MOVLW .8	;liczba bitw
		SUBWF i,W
		BTFSC C
		GOTO  et_zapis4 ;wysano wszystkie 8bitw

		BCF   RP0
		BCF   CLK	;CLK=0
		MOVF  temp,W	;temp1=temp
		MOVWF temp1
		MOVLW .1	;temp1=temp1&0x01
		ANDWF temp1,1
		DECFSZ temp1,W	;if(k==1)
		GOTO  et_zapis2
		BSF   DQ	;DQ=1
		GOTO  et_zapis3
et_zapis2	BCF   RP0	;else DQ1=0
		BCF   DQ
et_zapis3	BCF   RP0
		BSF   CLK	;CLK=1
		BCF   C		;bank0
		RRF   temp,1	;temp1=temp1>>1
		INCF  i,1
		GOTO  et_zapis1

et_zapis4	BTFSS bit9	;if(bit9==1)
		GOTO  et_zapis5
		BCF   RP0
		BCF   CLK	;CLK=0
		BTFSS znak
		BCF   DQ	;DQ=0
		BTFSC znak
		BSF   DQ	;DQ=1
		BSF   CLK	;CLK=1;
et_zapis5	BCF   RP0
		BSF   DQ	;DQ=1
		RETURN
;********************************************************************
;procedura konwersji sowa rejestru temperatury. W rejestrze temp warto
;temperatury po przecinku, w rejestrze dana wartoci bezwzgldna (modu)
;temperatury w formacie hex. 
;********************************************************************
wys_temp	MOVWF temp1		;zawarto W do temp1
		BTFSS znak		;czy znak=0
		GOTO  et_wyst		;rwny
		COMF  temp1,f		;temp1=~temp1 
		INCF  temp1,f		;temp1=temp1+1
		
et_wyst		BCF   C			;dana=temp1/2		
		RRF   temp1,W
		MOVWF dana		;w dana jednostki temperatury
		
		MOVLW .1		;temp=temp1%2	
		ANDWF temp1,W
		MOVWF ul		;
	
		BCF   C			;temp=temp*5
		RLF   ul,W
		ADDWF ul,W
		ADDWF ul,W
		ADDWF ul,1
		
		RETURN

;********************************************************************
;procedura konwersji zapisywanej temperatury
;********************************************************************
konw_term	MOVWF temp	;temp=W
		BTFSC znak
		GOTO  et_konw1	;temperatura ujemna
		BCF   C		;temp=temp*2
		RLF   temp,W	;W=temp
		GOTO  et_konw2
		
et_konw1	BCF   C          ;temp=temp*2
		RLF   temp,F	
		COMF  temp,F	 ;temp=~temp
		INCF  temp,W 	 ;temp=temp+1

et_konw2	RETURN

;********************************************************************
;procedura konwersji liczby hex umieszczonej w W z zakresu 0..63h na 
;format dziesietny 0..99
;liczba jednostek w rejestrze jedn, liczba dziesiatek w rejestrze dzies
;********************************************************************
_konw		movwf  dana		;	dzies=lhex/10;
		movwf temp1		
		clrf  i
		movlw .8
		movwf temp
k01		rlf   temp1,1
		rlf   i,1
		movlw .10
		subwf i,W
		btfss C
		goto  k02
		movlw .10
		subwf i,1
		bsf   C
		btfss C
k02		bcf   C
		rlf   dzies,1
		decfsz temp,1
		goto  k01
			;	temp1=dzies*10
		movlw .10
		movwf temp
		clrf temp1
k03		movf dzies,w
		addwf temp1,1
		decfsz temp,1
		goto k03

				;jedn=dana-temp1
		movf temp1,w
		subwf dana,w
		movwf jedn
		return


;--------------------------------------------------------------------
;procedura zapisu danej (RS=1) lub instrukcji (RS=0)
;zapisywana osimiobitowa dana znajduje si w rejestrze W
;--------------------------------------------------------------------
wrdata		bsf RS
		goto et_wr
wrcmd		bcf RS		;RS=0 rejestr instrukcji IR
et_wr		movwf TMP	;instrukcja do TMP
		bsf E		;E=1
		swapf TMP,f	;zamiana polbajtow

		movlw 0f0h	;jedynki na starszym polbajcie
		iorwf TMP,w	;f0h.OR.TMP->W
		movwf portb	;na portb starszy polbajt
		bcf E		;E=0
		nop
		nop
		bsf E		;E=1

		swapf TMP,w	;ponowna zamiana polbajtow i wpisnie do W
		iorlw 0f0h	;f0h.OR.W->W
		movwf portb
		bcf E		;E=0
		btfsc RS	;jaki jest stan linii RS
		goto et_wrdata	;zapis danej
		movlw .2
		call delay_msek	;opznienie 2msek dla instrukcji
		return
et_wrdata	call del_100	;opoznienie 100usek dla kodow znakow
		return
	
;--------------------------------------------------------------------
;porcedura inicjalizacji - interfejs 4bitowy
;--------------------------------------------------------------------
iniLCD		movlw .50	;opoznienie 50msek
		call delay_msek
		
		bcf RS		;RS=0 rejestr IR
		bsf E		;E=1
		movlw .3
		movwf portb

		bcf E		;E=0
		movlw .5	;opoznienie 5msek
		call delay_msek
		bsf E		;E=1
		nop		;ponowne wpisanie 3 modulu
		nop
		bcf E		;E=0
		call del_100	;czekaj 100usek
		bsf E		;E=1
		nop		;ponowne wpisanie 3 modulu
		nop
		bcf E		;E=0
		call del_100	;czekaj 100usek

		movlw 28h	;interfejs 4bitowy, 2linie znaki 5x7
		call wrcmd		
		movlw .8	;display off
		call wrcmd
		movlw .1	;display clear
		call wrcmd
		movlw .6	;entry mod
		call wrcmd
		movlw 0ch	;display on
		call wrcmd	
		return
;--------------------------------------------------------------------
;opnienie ok 100msek
;--------------------------------------------------------------------
del_100		bsf RP0		;bank1	
		movlw .8	;option_reg=8 preskaler do WDT 
		movwf option_reg
		bcf RP0		;bank0
		clrf tmr0	;zerowanie licznika TMR0

et_del100	movlw .50	;50->W (50*2usek=100usek)
		bcf RP0		;bank0
		subwf tmr0,w	;czy W = tmr0
		btfss C
		goto et_del100	;nie rowne
		return

		
;--------------------------------------------------------------------
;opnienie w milisekundach
;--------------------------------------------------------------------		
delay_msek	movwf msek
		bsf RP0		;bank1	
		movlw .1	;option_reg=1 preskaler do TMR0 
		movwf option_reg;podzal preskalera 1:4

et_del1		bcf RP0		;bank0
		clrf tmr0	;zerowanie licznika TMR0

et_del2		movlw .125	;125->W (125*4*2usek=1msek)
		bcf RP0		;bank0
		subwf tmr0,w	;czy W = tmr0
		btfss C
		goto et_del2	;nie rowne

		decfsz msek,f	;modyfikacja licznika milisekund
		goto et_del1
		return

		end	

