;********************************************************************
;                         Tomasz Jaboski                          *
;                 Mikrokontrolery PIC16F8x w praktyce               *
;                      Wydawnictwo BTC, www.btc.pl                  *
;********************************************************************

;procedury obslugi magistrali I2C
;Linia SDA - port RB0
;Linia SCL - port RB1
;program odczytuje w ptli stany linii wejciowych z ukadu PCF8574 i przesya je do 
;rejestru PORTA.
;Po wczeniu zasilania wszystkie linie PCF8574 s w stanie wysokim (wejcia)
;i nie trzeba wysyac do ukadu ffh
#include 16f84.h
bity	equ 10h
TRdat	equ 11h
PSW 	equ 12h
#define SDA	trisb,0
#define SCL	trisb,1
#define ACKF	PSW,0
		__config 	h'3ff1'
		org 0
		bsf RP0		;bank1
		movlw 0
		movwf trisa	;wszystkie linie wyjciowe
		movwf trisb	;wszystkie linie wyjciowe
		bcf RP0		;bank0
		movlw 0
		movwf portb	;wartoci poczatkowe
		movlw 0ffh	;rejestrw portw
		movwf porta
		bsf RP0
		bsf SDA		;SDA i SCL wejciowe TRISB<1> i TRISB<0> =1
		bsf SCL
		bcf RP0
		call starti2c	;wykonanie sekwencji satrtu
		movlw 41h	;adres PCF A0...A2=0
		movwf TRdat	;wysanie adrsu do PCF8574
		call i2ctxd

		
		bsf ACKF	;bit potwierdzenia
et		call i2crxd	;odczytanie danych z PCF8574
		movf TRdat,w
		movwf porta	;wysanie danych na PORTA
		goto et		;ptla nieskoczona
		
		;ten fragment wysya do PCF8574 55h
		movlw 55h
		movwf TRdat
		call i2ctxd  
		call stopi2c	;sekwencja stopu
		sleep

;--------------------------------------------------------------------
;podprogram wykonywania sekwencji startu
;--------------------------------------------------------------------
starti2c	bsf RP0		;bank1
		bcf SDA		;SDA wyjsciowy ->0
		nop
		nop
		nop
		nop
		nop
		bcf SCL		;SCL wyjsciowy ->0
		bcf RP0		;bank 0
		return
;--------------------------------------------------------------------
;podprogram sekwencji stopu
;--------------------------------------------------------------------
stopi2c		bsf RP0		;bank1
		bsf SCL		;SCL wejsciowy->1
		bcf RP0		;bank0
i2cwait		btfss SCL	;czekaj na zakonczenie wait state
				;sprawdzanie btfss 6,1 w banku 0 
		goto i2cwait	
		bsf RP0		;bank1
		bsf SDA		;SDA wejciowy ->1
		bcf RP0		;bank0
		return
;--------------------------------------------------------------------
;podprogram zapisu bajtu do SLAVE
;bajt do wyslania jest w TRdat
;po wyslaniu bit potwierdzenia jest w ACKF
;--------------------------------------------------------------------
i2ctxd		movlw .8	;licznik bitow do wyslania
		movwf bity
		bsf RP0		;bank 1	

i2ctx1		rlf TRdat,f	;najstarszy bit do C
		bcf SDA   	;SDA wyjsciowy ->0
		btfsc C		;sprawdz zawartosc C
		bsf SDA		;jezeli C=0 to pomin ten rozkaz
		bsf SCL		;SCL wejsciowy ->1
		bcf RP0		;bank0
i2ctx2		btfss SCL 	;btfss 6,1 w banku0 (rejestr portb)
				;czy SCL osiaga 1
		goto i2ctx2
		bsf RP0		;bank1
		bcf SCL		;SCL wyjsciowy ->0
		decfsz bity,f	;inkrementacja liczby bitow
		goto i2ctx1

		nop
		nop
		bsf SCL		;SCL wejsciowy ->1
		bcf RP0		;bank0
i2ctx3		btfss SCL
		goto i2ctx3
		bsf ACKF	;ACKF->1
		btfss SDA	
		bcf ACKF	;pomin ten rozkaz jezeli SDA=1
		bsf RP0		;bank1
		bcf SCL		;SCL wyjsciowy ->0
		bcf RP0		;bank0
		return		;
;--------------------------------------------------------------------
;podprogram odczytania bajtu ze SLAVE
;bajt odczytany umieszczony jest w TRdat
;bit potwierdzenia umieszczony jest w ACKF
;--------------------------------------------------------------------
i2crxd		bsf RP0		;bank1
		bsf SDA		;SDA wejsciowy ->1
		movlw .8
		movwf bity	;licznik odbieranych bitow
		bsf RP0		;bank1
		
i2crx1		bsf SCL		;SCL wejsciowy ->1
		bcf RP0		;bank0
i2crx2		btfss SCL	;czy SCL osiga 1(stan 'wait') 
		goto i2crx2
		rrf portb,w	;przesun SDA do TRdat
		rlf TRdat,f
		bsf RP0		;bank1
		bcf SCL		;SCL wyjciowy ->0
		decfsz bity,f	;inkrementacja licznika bitw
		goto i2crx1
		
		btfsc ACKF	;ustawienie bitu potwierdzenia
		bcf SDA		;SDA wyjsciowa ->0
		bsf SCL		;SCL wejsciowa ->1
		bcf RP0		;bank0
i2crx3		btfss SCL	;czy SCL osiga 1(stan 'wait')
		goto i2crx3
		bsf RP0		;bank1
		bcf SCL		;SCL wyjsciowy ->0
		bcf RP0		;bank0
		return
;--------------------------------------------------------------------

		END
