;********************************************************************
;                         Tomasz Jaboski                          *
;                 Mikrokontrolery PIC16F8x w praktyce               *
;                      Wydawnictwo BTC, www.btc.pl                  *
;********************************************************************

;wywietlenie napisu "www.btc.pl" na wywietlaczu LCD od adresu 0
;poaczenie moduu AVT z moduem wywietlacza pokazane jest na 
;rysunku 8.19
#include 16f84.h
msek		equ 10h
TMP		equ 11h
adres		equ 12h
#define E	porta,1
#define RS	porta,0

;********************************************************************
		__config 	h'3ff1'; slowo konfiguracyjne 
		org 0
		bsf RP0		;bank1
		movlw 0f0h
		movwf trisb	;RB7..RB4 wejsciowe, RB3..RB0 wyjciowe
		movwf trisa	;RA3..RA0 wyjciowe
		bcf RP0		;bank0
		call iniLCD	;interface 4bitowy

		movlw 80h	;adres 00
		call wrcmd
		call wys_strona ;wywietlenie napisu www.btc.pl
		sleep

;--------------------------------------------------------------------
;procedura wywietlenia napisu koczcego si znakiem koca (00h)
;kody znakw umieszczone s w tablicy kodow (dt......)
;--------------------------------------------------------------------
wys_strona	movlw .0	;adres poczatku w tablicy kodow
		movwf adres	
et_wys		call strona	;pobranie kodu z tablicy
		addlw 0		;sprawdzenie czy kod=0
		btfsc Z
		return		;kod=0 (kod konca)
		call wrdata	;wpisanie kodu do pamieci DD RAM
		incf adres,f	;modyfikacja adresu tablicy kodow (retlw)
		movf adres,w
		goto et_wys		

strona		addwf pcl,f
		dt "www.btc.pl",0

		
;--------------------------------------------------------------------
;procedura zapisu danej (RS=1) lub instrukcji (RS=0)
;zapisywana osimiobitowa dana znajduje si w rejestrze W
;--------------------------------------------------------------------
wrdata		bsf RS
		goto et_wr
wrcmd		bcf RS		;RS=0 rejestr instrukcji IR
et_wr		movwf TMP	;instrukcja do TMP
		bsf E		;E=1
		swapf TMP,f	;zamiana polbajtow

		movlw 0f0h	;jedynki na starszym polbajcie
		iorwf TMP,w	;f0h.OR.TMP->W
		movwf portb	;na portb starszy polbajt
		bcf E		;E=0
		nop
		nop
		bsf E		;E=1

		swapf TMP,w	;ponowna zamiana polbajtow i wpisnie do W
		iorlw 0f0h	;f0h.OR.W->W
		movwf portb
		bcf E		;E=0
		btfsc RS	;jaki jest stan linii RS
		goto et_wrdata	;zapis danej
		movlw .2
		call delay_msek	;opznienie 2msek dla instrukcji
		return
et_wrdata	call del_100	;opoznienie 100usek dla kodow znakow
		return
	
;--------------------------------------------------------------------
;porcedura inicjalizacji - interfejs 4bitowy
;--------------------------------------------------------------------
iniLCD		movlw .50	;opoznienie 50msek
		call delay_msek
		
		bcf RS		;RS=0 rejestr IR
		bsf E		;E=1
		movlw .3
		movwf portb

		bcf E		;E=0
		movlw .5	;opoznienie 5msek
		call delay_msek
		bsf E		;E=1
		nop		;ponowne wpisanie 3 modulu
		nop
		bcf E		;E=0
		call del_100	;czekaj 100usek
		bsf E		;E=1
		nop		;ponowne wpisanie 3 modulu
		nop
		bcf E		;E=0
		call del_100	;czekaj 100usek

		movlw 28h	;interfejs 4bitowy, 2linie znaki 5x7
		call wrcmd		
		movlw .8	;display off
		call wrcmd
		movlw .1	;display clear
		call wrcmd
		movlw .6	;entry mod
		call wrcmd
		movlw 0ch	;display on
		call wrcmd	
		return
;--------------------------------------------------------------------
;opnienie ok 100msek
;--------------------------------------------------------------------
del_100		bsf RP0		;bank1	
		movlw .8	;option_reg=8 preskaler do WDT 
		movwf option_reg
		bcf RP0		;bank0
		clrf tmr0	;zerowanie licznika TMR0

et_del100	movlw .50	;50->W (50*2usek=100usek)
		bcf RP0		;bank0
		subwf tmr0,w	;czy W = tmr0
		btfss C
		goto et_del100	;nie rowne
		return

		
;--------------------------------------------------------------------
;opnienie w milisekundach
;--------------------------------------------------------------------		
delay_msek	movwf msek
		bsf RP0		;bank1	
		movlw .1	;option_reg=1 preskaler do TMR0 
		movwf option_reg;podzal preskalera 1:4

et_del1		bcf RP0		;bank0
		clrf tmr0	;zerowanie licznika TMR0

et_del2		movlw .125	;125->W (125*4*2usek=1msek)
		bcf RP0		;bank0
		subwf tmr0,w	;czy W = tmr0
		btfss C
		goto et_del2	;nie rowne

		decfsz msek,f	;modyfikacja licznika milisekund
		goto et_del1
		return

		end	
